package de.fhhannover.inform.trust.ifmapj_examples;

import java.util.Collection;

import de.fhhannover.inform.trust.ifmapj.IfmapJ;
import de.fhhannover.inform.trust.ifmapj.IfmapJHelper;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapException;
import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;
import de.fhhannover.inform.trust.ifmapj.identifier.IdentifierFactory;
import de.fhhannover.inform.trust.ifmapj.identifier.IpAddress;
import de.fhhannover.inform.trust.ifmapj.messages.PublishRequest;
import de.fhhannover.inform.trust.ifmapj.messages.PublishUpdate;
import de.fhhannover.inform.trust.ifmapj.messages.RequestFactory;
import de.fhhannover.inform.trust.ifmapj.messages.ResultItem;
import de.fhhannover.inform.trust.ifmapj.messages.SearchRequest;
import de.fhhannover.inform.trust.ifmapj.messages.SearchResult;
import de.fhhannover.inform.trust.ifmapj.metadata.StandardIfmapMetadataFactory;

/**
 * This example publishes a link with two identical identifiers. In graph theory,
 * an edge which starts and ends at the same vertex is called a loop. We try to
 * publish a metadata such that a loop is created in the MAP graph.
 * 
 * The result though, is probably MAPS dependent. 
 * 
 * This example further shows how code can be simplified by using the "convenience
 * factory methods". This is shown with, for example,
 * createPublishUpdate(ip, ip, mF.createIpMac()), which creates a
 * {@link PublishUpdate} object, containing a Link and a single metadata
 * element (ip-mac metadata).
 */
public class LoopLink {
	
	public void run(String[] args) throws InitializationException {

		System.out.println("====== RUNNING LOOP EXAMPLE ======");
	
		// Use basic authentication.
		SSRC ssrc = IfmapJ.createSSRC(
				Config.BASIC_AUTH_SERVER_URL,
				Config.BASIC_AUTH_USER,
				Config.BASIC_AUTH_PASSWORD,
				IfmapJHelper.getTrustManagers(
						getClass().getResourceAsStream(Config.TRUST_STORE_PATH),
						Config.TRUST_STORE_PASSWORD));	
		

		// To create simple requests, we need some factories.
		RequestFactory rF = IfmapJ.createRequestFactory();
		IdentifierFactory iF = IfmapJ.createIdentifierFactory();
		StandardIfmapMetadataFactory mF = IfmapJ.createStandardMetadataFactory();
		
		IpAddress ip = iF.createIp4("192.168.0.1");
		// Create a PublishUpdate object containing a link. Note: It doesn't
		// really make sense to publish ip-mac metadata between to IP addresses.
		PublishUpdate pu = rF.createPublishUpdate(ip, ip, mF.createIpMac());
		PublishRequest pr = rF.createPublishReq(pu);
	
		// Create a SearchRequest. null means not specified. So this will search
		// the MAPS beginning from "192.168.0.1" with a max-depth of 10.
		SearchRequest sr = rF.createSearchRequest(null, 10, null, null, null, ip);

		try {
			
			// create a new session
			System.out.print("newSession ... ");
			ssrc.newSession();
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// publish the loop
			System.out.print("publish update ... ");
			ssrc.publish(pr);
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();

			// search for the link
			System.out.print("search ... ");
			SearchResult sres = ssrc.search(sr);
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// print out the result.
			print(sres);
			IfmapJExamples.sleepSomeTime();
			
			// end the session
			System.out.print("endSession ... ");
			ssrc.endSession();
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
		// catch errors that may occur:
		// - If the server replies with an ErrorResult, a IfmapErrorResult
		//    is thrown.
		// - Other failures, for example due to network outage result in
		//   an IfmapException.
		} catch (IfmapException e) {
			System.err.println(e.getDescription());
		} catch (IfmapErrorResult e) {
			System.err.println(e);
		}
	}

	/**
	 * Helper to print out all Result items...
	 * 
	 */
	private void print(SearchResult sres) {
		System.out.println("---- print SearchResult -------------------------");
		Collection<ResultItem> ris = sres.getResultItems();
		int i = 0;
		for (ResultItem ri : ris) {
			System.out.println("Item " + ++i + ") {" + ri.getIdentifier1()
					+ ", " + ri.getIdentifier2() + ", #metadata=" 
					+ ri.getMetadata().size() + "}");
		}
		System.out.println("-------------------------------------------------");
	}
}
