package de.fhhannover.inform.trust.ifmapj_examples;

import de.fhhannover.inform.trust.ifmapj.IfmapJ;
import de.fhhannover.inform.trust.ifmapj.IfmapJHelper;
import de.fhhannover.inform.trust.ifmapj.binding.IfmapStrings;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapException;
import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;
import de.fhhannover.inform.trust.ifmapj.identifier.IdentifierFactory;
import de.fhhannover.inform.trust.ifmapj.identifier.IpAddress;
import de.fhhannover.inform.trust.ifmapj.identifier.MacAddress;
import de.fhhannover.inform.trust.ifmapj.messages.MetadataLifetime;
import de.fhhannover.inform.trust.ifmapj.messages.PublishDelete;
import de.fhhannover.inform.trust.ifmapj.messages.PublishRequest;
import de.fhhannover.inform.trust.ifmapj.messages.RequestFactory;
import de.fhhannover.inform.trust.ifmapj.metadata.StandardIfmapMetadataFactory;

/**
 * This example shows how the creation of publish requests can be condensed
 * to a few lines. If no lifetime is given for the creation of a PublishUpdate
 * object, it's by default {@link MetadataLifetime#session}.
 * 
 * It is also shown how "lifetime forever" metadata can be published.
 */
public class PublishExample2 {
	
	
	private RequestFactory rf = IfmapJ.createRequestFactory();
	private IdentifierFactory ifac = IfmapJ.createIdentifierFactory();
	private StandardIfmapMetadataFactory mf = IfmapJ.createStandardMetadataFactory();
	private IpAddress ip = ifac.createIp4("192.168.0.1");
	private MacAddress mac = ifac.createMac("aa:bb:cc:dd:ee:ff");

	public void run(String args[]) throws InitializationException {
		

		System.out.println("====== RUNNING PUBLISH EXAMPLE 2 ======");
		
		SSRC ssrc = IfmapJ.createSSRC(
				Config.BASIC_AUTH_SERVER_URL,
				Config.BASIC_AUTH_USER,
				Config.BASIC_AUTH_PASSWORD,
				IfmapJHelper.getTrustManagers(getClass().getResourceAsStream(
						Config.TRUST_STORE_PATH),
						Config.TRUST_STORE_PASSWORD));
		
		try {
		
			System.out.print("newSession ... ");
			ssrc.newSession();
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// Publish ip-mac metadata between 192.168.0.1 and aa:bb:cc:dd:ee:ff
			System.out.print("publish update (session) ... ");
			ssrc.publish(rf.createPublishReq(rf.createPublishUpdate(
					ip, mac, mf.createIpMac())));
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
	
			// Delete it again, see preparePublishDelete()...
			System.out.print("publish delete ... ");
			ssrc.publish(preparePublishDelete());
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
	
			// Publish the same metadata, this time using explicit lifetime forever
			System.out.print("publish update (forever) ... ");
			ssrc.publish(rf.createPublishReq(rf.createPublishUpdate(
					ip, mac, mf.createIpMac(), MetadataLifetime.forever)));
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// Publish the same metadata, this time using explicit lifetime session
			System.out.print("publish update (session) ... ");
			ssrc.publish(rf.createPublishReq(rf.createPublishUpdate(
					ip, mac, mf.createIpMac(), MetadataLifetime.session)));
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
		
			// Ending the session removes all metadata with lifetime=session.
			// Note: If we directly call newSession(), the same behavior is expected,
			// but it wouldn't be as clear.
			System.out.print("endSession ... ");
			ssrc.endSession();
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// Create a new session.
			System.out.print("newSession ... ");
			ssrc.newSession();
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// Delete the forever metadata. "Forever metadata" stays across multiple
			// sessions.
			System.out.print("publish delete ... ");
			ssrc.publish(preparePublishDelete());
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// Now, end the session, hoping there's no metadata left.
			System.out.print("endSession ... ");
			ssrc.endSession();
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
		
		// catch errors that may occur:
		// - If the server replies with an ErrorResult, a IfmapErrorResult
		//    is thrown.
		// - Other failures, for example due to network outage result in
		//   an IfmapException.
		} catch (IfmapException e) {
			System.err.println(e.getDescription());
		} catch (IfmapErrorResult e) {
			System.err.println(e);
		}
	}

	private PublishRequest preparePublishDelete() {
		PublishDelete del = rf.createPublishDelete(ip, mac, "meta:ip-mac");
		
		// As the meta: prefix in the filter string is used, the declaration
		// of this namespace needs to be made. If this isn't done, a MAPS will
		// either complain about it, or won't delete the appropriate metadata.
		del.addNamespaceDeclaration(IfmapStrings.STD_METADATA_PREFIX,
				IfmapStrings.STD_METADATA_NS_URI);
		return rf.createPublishReq(del);
	}
}
